100 rem     Free memory button
102 rem Requires button frame and my button toolkit
104 rem V0.01 pjwitte October 12th 2016
106 rem V0.02, pjw, March 19th 2018, added border highlight. EasyPtr only!
108 rem V0.03, pjw, 2022 Oct 02, command line options, bar display (SMSQ/E only)
110 :
112 ext_fn 'BUTUSE%', 'DETAB$', 'VALID%'
114 :
116 rem     Example QLib directives
118 rem $$asmb=WIN1_uti_but_fre_BUTFREE_BIN,0,10
120 rem rem $$asmb=win1_rxt_easyptr_cde,4,82
122 remark $$chan=3
124 remark $$stak=256
126 :
128 rem     Example Turbo directives
130 rem rem %%WIN1_uti_but_fre_BUTFREE_BIN,0,10
132 rem rem %%win1_rxt_easyptr_cde,4,82
134 turbo_objdat 3
136 turbo_taskn 'BUTFREE'
138 :
140 rem     Wman constants
142 sp_buthigh%     = 548: rem Button highlight
144 sp_butbd%       = 549: rem Button border
146 sp_butbg%       = 550: rem Button background
148 sp_butfg%       = 551: rem Button foreground
150 sp_butmg%       = 552: rem Button middleground
152 sp_errbg%       = 556: rem Error message background
154 sp_errfg%       = 557: rem Error message foreground
156 sp_errmg%       = 558: rem Error message middleground
158 :
160 rem     Window Metrics, etc
162 rem xo%  = 11 * 6 + 4: rem will be button x-size (unless CMD$)
164 y%       = 14:         rem will be button y-size
166 tio%     = 50:         rem loop timeout == 1s
168 minm     = 900000:     rem min free memory for warning
170 tvi%     = 33:         rem termination vector in window (EasyPtr)
172 tvo%     = 17:         rem termination vector out of window
174 :
176 quote$  = '"' & "'"
178 :
180 rem     Initialise
182 job_name 'BUTFREE':    rem SBASIC only
184 spjob -1, 126:         rem SBASIC or toolkit required
186 :
188 ert GetCmd(CMD$)
190 :
192 x% = wid%: xo% = x%: yo% = y%
194 ert butuse%(xo%, yo%):              rem Give size; return position
196 xl% = xo% + x%: yl% = yo% + y%:     rem Button limits
198 xs% = x% - 4: ys% = y% - 2:         rem Inner dimensions
200 :
202 sp_jobpal -1, pal
204 :
206 ch% = fopen("con")
208 window#ch%; x%, y%, xo%, yo%
210 outl#ch%
212 wm_paper#ch%; sp_butbg%
214 wm_border#ch%; 1, sp_butbd%: cls#ch%
216 spjob -1, 126: rem Set priority so clock will be picked with Button_Pick
218 lastm = -1: xp% = -1: yp% = xp%
220 tram = int(peek_l(!! $20) / $100000 + 0.5) * $100000: rem Total RAM (incl OS memory)
222 :
224 rem     Action loop
226 rep out_loop
228
230  Free
232
234  rdpt#ch%; tvi%, xp%, yp%, tio%
236
238  if xp% > xl% or xp% < xo%: next out_loop
240  if yp% > yl% or yp% < yo%: next out_loop
242
244 rem Pointer inside window now
246
248  wm_border#ch%; 1, sp_buthigh%
250
252  rep in_loop
254
256   Free
258
260   rdpt#ch%; tvo%, xp%, yp%, tio%
262   if (tvo% div 256) = 27: exit out_loop
264
266   if xp% > xl% or xp% < xo%: exit in_loop
268   if yp% > yl% or yp% < yo%: exit in_loop
270
272  end rep in_loop
274
276 rem Pointer moved outside window now
278
280  wm_border#ch%; 1, sp_butbd%
282
284 endrep out_loop
286
288 quit
290 :
292 def proc Free
294 freem = free_mem
296 if freem <> lastm then
298  if freem < minm then
300   beep 5000, 200
302   wm_paper#ch%; sp_errbg%
304   ink% = sp_errfg%
306  else
308   wm_paper#ch%; sp_butbg%
310   ink% = sp_butfg%
312  endif
314
316  cls#ch%
318  cursor#ch%; (xs% - 60) / 2, 1
320  wm_ink#ch%; ink%
322  bput#ch%; cdec$(freem / 1024, 7, 0), ' Kb'
324  if bar then
326   over#ch%; -1
328   if mid: ink% = sp_butmg%
330   wm_block#ch%; xs% * freem / tram, ys%, 0, 0, ink%
332   over#ch%; 0
334  endif
336  lastm = freem
338 endif
340 enddef Free
342 :
344 :
346 rem + ------------------------------------------------------------------------ +
348 rem |<                                Command Line                            >|
350 rem + ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ +
352 rem |                    Parses command line and sets defaults                 |
354 rem |                                                                          |
356 rem |  /B - 1 => display bar [default], 0 => dont                              |
358 rem |  /M - 1 => Use button middleground Wman colour, 0 => dont [default]      |
360 rem |  /P - Palette 0 [default] .. 3                                           |
362 rem |  /W - Button Width 72 [min, default] .. 204                              |
364 rem |                                                                          |
366 rem | Dependencies: DETAB$ VALID%                                              |
368 rem | GLOBal quote$, bar, mid, pal, wid%                                       |
370 rem + ------------------------------------------------------------------------ +
372 rem | V0.01, pjw, 2019 Jul 01, base version                                    |
374 rem | V0.01, pjw, 2022 Oct 02, FREE_MEM version                                |
376 rem + ------------------------------------------------------------------------ +
378 :
380 def fn GetCmd(cl$)
382 loc p%, t$(50)
384 :
386 rem     Defaults
388 pal  = 0
390 bar  = 1
392 mid  = 0
394 wid% = 72:              rem 11 * 6 + 4 (+2)
396 if len(cl$) = 0: ret 0
398 :
400 rem     Palette
402 t$ = Getent$("P")
404 if len(t$) = 1 then
406  if t$ instr '0123': pal = t$
408 endif
410 :
412 rem     Bar
414 t$ = Getent$("B")
416 if len(t$) = 1 then
418  if t$ instr '01': bar = t$
420 endif
422 :
424 rem     Middleground
426 t$ = Getent$("M")
428 if len(t$) = 1 then
430  if t$ instr '01': mid = t$
432 endif
434 :
436 rem     Width
438 t$ = Getent$("W")
440 if valid%(3, t$) then
442  p% = t$
444  sel on p%: = 70 to 204: wid% = p%
446  wid% = (wid% + 3) && -4:           rem Divisible by 4
448 endif
450 :
452 ret 0
454 enddef GetCmd
456 :
458 def fn Getent$(c$)
460 rem Caller's cl$, p%, t$
462 rem Given the character get the entry
464 rem /C [<spaces>] ["|'] <entry> ["|'] <space> | <space> /C+1 | <eol>
466 :
468 p% = '/' & c$ instr cl$
470 if p% = 0: ret ''
472 t$ = detab$(cl$(p% + 2 to len(cl$)))
474 p% = ' /' instr t$
476 if p% > 0: t$ = t$(1 to p%): else: p% = len(t$)
478 for p% = len(t$) to 1 step -1
480  if not t$(p%) instr ' /': exit p%
482 endfor p%
484 :
486 t$ = t$(1 to p%)
488 if len(t$) > 1 then
490  p% = t$(1) instr quote$
492  if p% then
494   if (t$(len(t$)) instr quote$) = p% then
496    t$ = t$(2 to len(t$) - 1)
498   endif: endif: endif
500 ret t$
502 enddef Getent$
504 :
506 :
